% This is a small example to demonstrate how the q-SPICE implementation
% works. 

%% Johan Swrd, 20160921

%% Clear variables and tidy up
clearvars
close all
clc
warning('off');
%%

%% Signal settings
% Number of samples
N = 50; 
% Time vector
t = (1:N)';
% Number of frequency parameters (dictionart size)
M =1000; 
% Frequency vector of candidate frequencies
freqVec = linspace(0,1-1/M,M); 
% Number of modes
K = 3; 
% Dictionary NxM
B = zeros(N,M); 
for iM=1:M
    a = exp(2i*pi*freqVec(iM)*t);
    % Make sure that each column has unit norm 
    B(:,iM) = a/norm(a); 
end
% Dictionary for the noise
I = eye(N);
% The full SPICE dictionary
A = [B I]; 
% SNR-level as SNR = 10 log_10(signal_power/noise_power)
SNR = 10; 
%%
% Randomly draw frequencies that are separated at least 1/2N from
% eachother.
freqs = sort(rand(K,1)); 
while min(diff(freqs)) <1/2/N || min(freqs) <1/2/N || max(freqs) >1-1/2/N
    freqs = sort(rand(K,1));
end
%% 
% Generate amplitudes with unit magnitude and random phase
alpha = [1 1 1]'.*exp(1i*pi*rand(K,1));
% Generate signal
y=zeros(N,1);
for iFreqs = 1:K
    a = exp(2i*pi*freqs(iFreqs)*t);
    a = a/norm(a);
    y = y + alpha(iFreqs)*a;
end
yorg = y;
% Add noise to the signal
sigma = sqrt(norm(y)^2/length(y(:))*10^(-SNR/10));
noise = 1i*randn(N,1)+randn(N,1);
y = yorg+sigma*noise/sqrt(2);
%%

% Select which q (norm) for q-SPICE
q = 2; 
% Set the precision for the algorithm
precision = 1e-4;
tic
[p,sigmaEst,R,iterEnd]=q_SPICE(y,B,q,precision);
toc
% Estimate the correpsonding amplitudes
x_implemented = zeros(M,1);
for m = 1:M
    x_implemented(m) = p(m)*(B(:,m)'*(R\y));
end
%% Plot the result
figure
stem(freqVec,abs(x_implemented))
xlabel('Frequency')
ylabel('Magnitude')
hold all
plot(freqs,abs(alpha),'*')
hold off
%%
% Find the frequencies
[pks,locs] = findpeaks(abs(x_implemented));
freqEst = freqVec(locs);
disp(['True frequencies are ' num2str(freqs(:)')])
disp(['Estimated frequencies are ' num2str(freqEst(:)')])
